import ast
from io import BytesIO
from shlex import split
from tokenize import COMMENT
from tokenize import tokenize

from astpretty import pprint

code1 = """
print(2)
print(2)
"""
code2 = """
async def f(request):
    print(2)
"""


class NumberChanger(ast.NodeTransformer):
    """Changes all number literals to 42."""

    def visit_Module(self, node):
        super().generic_visit(node)

        if isinstance(node, ast.Module):
            wrapper = ast.AsyncFunctionDef(
                name="do_request",
                decorator_list=[],
                args=ast.arguments(
                    posonlyargs=[],
                    kwonlyargs=[],
                    defaults=[],
                    kw_defaults=[],
                    args=[ast.arg(arg="request")],
                ),
            )
            wrapper.body = node.body
            node.body = [wrapper]
            return node
        if not isinstance(node, ast.Constant) or not isinstance(node.value, int):
            return node

        return ast.Constant(value=42)


# pprint(ast.parse(code1))
# pprint(ast.parse(code2))
# pprint(a)
modded1 = ast.fix_missing_locations(NumberChanger().visit(ast.parse(code1)))
# pprint(modded1)
modded2 = ast.parse(code2)
# pprint(modded2)
# print(modded1 == modded2)
# print("--------------")
# print(ast.unparse(modded1))
# print(ast.unparse(modded2))

# pprint(
#     ast.parse(
#         """
# return dark_star_templates.TemplateResponse("{template_path}", locals())
# """
#     )
# )
# code = '''
# print('')
# """adfa
# fadf
# a
# fadf
# ad
# """
# '''
# pprint(ast.parse(code))

code = """


# methods="GET, POST" name="this is my name"


r = request
"""
g = tokenize(BytesIO(code.strip().encode("utf-8")).readline)


for toknum, tokval, (srow, scol), *_ in g:
    if toknum == COMMENT:
        print(tokval, srow)
        for s in split(tokval):
            print(s)

pprint(ast.parse(code))



import ast
import os
from os import PathLike
from pathlib import Path
import typing

import jinja2
from jinja2.environment import Environment
from jinja2.exceptions import TemplateNotFound
from jinja2.loaders import FileSystemLoader as JinjaFileSystemLoader
from jinja2.loaders import split_template_path
from jinja2.utils import open_if_exists
from starlette.templating import Jinja2Templates as StarletteJinja2Templates

# @contextfunction renamed to @pass_context in Jinja 3.0, to be removed in 3.1
if hasattr(jinja2, "pass_context"):
    pass_context = jinja2.pass_context
else:  # pragma: nocover
    pass_context = jinja2.contextfunction


class TemplateExtractor(ast.NodeVisitor):
    """Makes our bare files into functions"""

    def visit_Constant(self, node):
        super().generic_visit(node)

        self.value = node.value


class FileSystemLoader(JinjaFileSystemLoader):
    def get_source(
        self, environment: Environment, template: str
    ) -> typing.Tuple[str, str, typing.Callable[[], bool]]:

        pieces = split_template_path(template)

        for searchpath in self.searchpath:
            filename = os.path.join(searchpath, *pieces)

            f = open_if_exists(filename)

            if f is None:
                continue
            try:
                contents = f.read().decode(self.encoding)
            finally:
                f.close()

            if Path(filename).suffix == ".py":
                extractor = TemplateExtractor()
                extractor.visit(ast.parse(contents))
                contents = extractor.value

            mtime = os.path.getmtime(filename)

            def uptodate() -> bool:
                try:
                    return os.path.getmtime(filename) == mtime
                except OSError:
                    return False

            return contents, filename, uptodate
        raise TemplateNotFound(template)


class Jinja2Templates(StarletteJinja2Templates):
    def _create_env(
        self, directory: typing.Union[str, PathLike], **env_options: typing.Any
    ) -> "jinja2.Environment":
        @pass_context
        def url_for(context: dict, name: str, **path_params: typing.Any) -> str:
            request = context["request"]
            return request.url_for(name, **path_params)

        loader = FileSystemLoader(directory)
        env_options.setdefault("loader", loader)
        env_options.setdefault("autoescape", True)

        env = jinja2.Environment(**env_options)
        env.globals["url_for"] = url_for
        return env



import ast
from hashlib import md5
from io import BytesIO
from pathlib import Path
import shlex
from tokenize import COMMENT
from tokenize import tokenize
import typing

from starlette.applications import Starlette
from starlette.middleware import Middleware
from starlette.requests import Request
from starlette.responses import Response
from starlette.routing import BaseRoute
from starlette.routing import Mount
from starlette.routing import Route
from starlette.staticfiles import StaticFiles

from .templating import Jinja2Templates


dark_star_templates = None


class FunctionAdder(ast.NodeTransformer):
    """Makes our bare files into functions"""

    def __init__(self, template_path, function_name, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.function_name = function_name
        self.template_path = template_path
        self.new_return = ast.parse(
            f"""return dark_star_templates.TemplateResponse("{self.template_path}", locals())"""
        )

    def visit_Module(self, node):
        super().generic_visit(node)

        wrapper = ast.AsyncFunctionDef(
            name=self.function_name,
            decorator_list=[],
            args=ast.arguments(
                posonlyargs=[],
                kwonlyargs=[],
                defaults=[],
                kw_defaults=[],
                args=[ast.arg(arg="request")],
            ),
        )
        wrapper.body = node.body
        wrapper.body.extend(self.new_return.body)
        node.body = [wrapper]
        return node


def get_options(code):
    tokens = tokenize(BytesIO(code.strip().encode("utf-8")).readline)
    for toknum, tokval, (srow, scol), *_ in tokens:
        if srow > 1:
            return {}
        if toknum == COMMENT:
            _, *options = shlex.split(tokval)
            break
    route_options = {}
    if options:
        for option in options:
            key, _, value = option.partition("=")
            if key == "methods":
                route_options[key] = [x.strip() for x in value.split(",")]
            elif key == "name":
                route_options[key] = value
    return route_options


class DarkStar(Starlette):
    def __init__(
        self,
        routes_path: typing.Union[str, Path] = "routes",
        debug: bool = False,
        routes: typing.Sequence[BaseRoute] = [],
        static_directory: str = "static",
        middleware: typing.Sequence[Middleware] = None,
        exception_handlers: typing.Mapping[
            typing.Any,
            typing.Callable[
                [Request, Exception], typing.Union[Response, typing.Awaitable[Response]]
            ],
        ] = None,
        on_startup: typing.Sequence[typing.Callable] = None,
        on_shutdown: typing.Sequence[typing.Callable] = None,
        lifespan: typing.Callable[["Starlette"], typing.AsyncContextManager] = None,
    ) -> None:

        global dark_star_templates
        dark_star_templates = Jinja2Templates(routes_path)

        self.template_env = dark_star_templates.env

        path_routes = self._collect_routes(routes_path)

        if not any(
            type(route) == Mount and type(route.app) == StaticFiles for route in routes
        ):
            routes.append(Mount("/static/", StaticFiles(directory=static_directory)))

        super().__init__(
            debug,
            path_routes + routes,
            middleware,
            exception_handlers,
            on_startup,
            on_shutdown,
        )

    def _collect_routes(self, routes_path) -> typing.Sequence[BaseRoute]:
        routes = []

        for path in Path(routes_path).rglob("*.py"):
            if path.is_file():
                python = path.read_text()
                function_name = f"ds_{md5(str(path).encode()).hexdigest()}"

                modded_function = ast.fix_missing_locations(
                    FunctionAdder(path.relative_to(routes_path), function_name).visit(
                        ast.parse(python)
                    )
                )

                exec(compile(modded_function, f"{path}", "exec"), globals())

                route_options = get_options(python)

                if path.relative_to(routes_path) == Path("index.py"):
                    if "name" not in route_options:
                        route_options["name"] = "index"
                    routes.append(Route("/", globals()[function_name], **route_options))
                else:
                    routes.append(
                        Route(
                            f"/{path.relative_to(routes_path).with_suffix('')}/",
                            globals()[function_name],
                            **route_options,
                        )
                    )

        return routes



from pathlib import Path

import click
from jinja2 import Template
from starlette.routing import compile_path

APP_TEMPLATE = """\
from darkstar.applications import DarkStar

app = DarkStar()
"""

INDEX_TEMPLATE = '''
"""
<html data-theme="dark">
  <head>
    <title>Dark Star</title>
    {%- if htmx %}
    <script src="https://unpkg.com/htmx.org@latest"></script>
    <script src="https://unpkg.com/hyperscript.org@latest"></script>{% endif %}
    <link rel="stylesheet" href="https://classless.de/classless.css">
    <link rel="stylesheet" href="https://classless.de/addons/themes.css">
  </head>
  <body>
  {%- raw %}
    {% block content %}
    <h1>Welcome to Dark Star</h1>
    <p>Use the <code>new-route</code> command to create your first route</p>
    <p>See <a href="https://lllama.github.io/dark-star">the docs</a> for more information.</p>
    {% endblock %}
  {% endraw -%}
  </body>
</html>
"""
'''

TEMPLATE_TEMPLATE = '''\
{{param_text}}
"""
{% raw %}
{% extends 'index.py' %}
{% block content %}
{% endblock %}
{% endraw %}
"""
'''


@click.group("cli")
@click.pass_context
def cli(ctx):
    """Dark Star - a web framework based on Starlette"""
    pass


@cli.command("create-app")
@click.argument("directory", type=click.Path())
@click.option("--htmx/--no-htmx", help="Include htmx script in index.py", default=False)
def create_app(directory, htmx):
    path = Path(directory)

    routes_dir = path / "routes"
    print(f"Creating {routes_dir}")
    routes_dir.mkdir(exist_ok=True, parents=True)

    static_dir = path / "static"
    print(f"Creating {static_dir}")
    static_dir.mkdir(exist_ok=True, parents=True)

    index_path = path / "routes" / "index.py"
    if not index_path.exists():
        print(f"Creating {index_path}")
        index_path.write_text(Template(INDEX_TEMPLATE).render(htmx=htmx))
    app_file = path / "app.py"
    if not app_file.exists():
        print(f"Creating {app_file}")
        app_file.write_text(Template(APP_TEMPLATE).render())


@cli.command("new-route")
@click.argument("path", type=click.Path())
def new_route(path):

    route_path = Path("routes") / path.lstrip("/").rstrip("/")
    route_path = route_path.with_suffix(".py")
    if not route_path.exists():
        print(f"Adding new route {route_path}")
        route_path.parent.mkdir(parents=True, exist_ok=True)
        *_, params = compile_path(str(route_path))
        param_text = "\n".join(
            f'{key} = request.path_params["{key}"]' for key in params.keys()
        ).strip()
        route_path.write_text(Template(TEMPLATE_TEMPLATE).render(param_text=param_text))


def main():
    cli(prog_name="cli")


if __name__ == "__main__":
    main()



#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['darkstar']

package_data = \
{'': ['*']}

package_dir = \
{'': 'src'}

install_requires = \
['starlette', 'starlette-htmx', 'jinja2', 'click']

entry_points = \
{'console_scripts': ['darkstar = darkstar.__main__:main']}

setup(name='darkstar',
      version='0.1.5',
      description='Dark Star',
      author=None,
      author_email='Felix Ingram <f.ingram@gmail.com>',
      url=None,
      packages=packages,
      package_data=package_data,
      package_dir=package_dir,
      install_requires=install_requires,
      entry_points=entry_points,
      python_requires='>=3.6',
     )
